/* --------------------------------------------- */
/*  komekame touch sensor library                */
/*  Ver 0.10                                     */
/*                       Programed by rerofumi   */
/* --------------------------------------------- */

/* ----------------------- */
/*  include                */
/* ----------------------- */
#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

/* ----------------------- */
/*  define                 */
/* ----------------------- */

/* ----------------------- */
/*  global value           */
/* ----------------------- */

/**
 * charge time
 */
unsigned short  m_riseCounter[8];

/**
 * working flag
 */
char m_bTouchWorking;

/**
 * reading port counter
 */
char m_nReadingPort;


/* ----------------------- */
/*  interrupt              */
/* ----------------------- */

/* ---------------------------------- */
/**
 * Counter Terminate
 */
#pragma interrupt_handler mkIntCounterTimeover
void mkIntCounterTimeover(void)
{
	unsigned char  dm0, dm1;
	/* is working? */
	if (0 == m_bTouchWorking) {
		return;
	}
	
	/* timeout goto next port */
	m_riseCounter[m_nReadingPort] = 0xffff;
	m_nReadingPort = (m_nReadingPort + 1) & 3;
	switch(m_nReadingPort) {
		case 0:
			dm0 = 0b11111101;
			dm1 = 0b00000010;
			break;
		case 1:
			dm0 = 0b11111011;
			dm1 = 0b00000100;
			break;
		case 2:
			dm0 = 0b11110111;
			dm1 = 0b00001000;
			break;
		case 3:
			dm0 = 0b11101111;
			dm1 = 0b00010000;
			break;
		default:
			dm0 = 0xff;
			dm1 = 0x00;
	}
	/* refresh */
	M8C_SetBank1;
	PRT0DM0 = 0xff;
	PRT0DM1 = 0x00;
	M8C_SetBank0;
	PRT0DM2 = 0x00;
	PRT0DR = 0x00;
	/* set intrrupt */
	PRT0IE = dm1;
	/* counter reset */
	Counter16_Charge_WritePeriod(48000);
	/* charge start */
	M8C_SetBank1;
	PRT0DM0 = dm0;
	PRT0DM1 = dm1;
	M8C_SetBank0;
	PRT0DM2 = 0x00;
}

/* ---------------------------------- */
/**
 * GPIO Intrrupt
 */
#pragma interrupt_handler mkIntPinRise
void mkIntPinRise(void)
{
	unsigned char  dm0, dm1;
	/* is working? */
	if (0 == m_bTouchWorking) {
		return;
	}
	
	/* read counter */
	Counter16_Charge_Stop();
	m_riseCounter[m_nReadingPort] = Counter16_Charge_wReadCounter();

	/* goto next port */
	m_nReadingPort = (m_nReadingPort + 1) & 3;
	switch(m_nReadingPort) {
		case 0:
			dm0 = 0b11111101;
			dm1 = 0b00000010;
			break;
		case 1:
			dm0 = 0b11111011;
			dm1 = 0b00000100;
			break;
		case 2:
			dm0 = 0b11110111;
			dm1 = 0b00001000;
			break;
		case 3:
			dm0 = 0b11101111;
			dm1 = 0b00010000;
			break;
		default:
			dm0 = 0xff;
			dm1 = 0x00;
	}
	/* refresh */
	M8C_SetBank1;
	PRT0DM0 = 0xff;
	PRT0DM1 = 0x00;
	M8C_SetBank0;
	PRT0DM2 = 0x00;
	PRT0DR = 0x00;
	/* set intrrupt */
	PRT0IE = dm1;
	/* counter reset */
	Counter16_Charge_WritePeriod(48000);
	Counter16_Charge_Start();
	/* charge start */
	M8C_SetBank1;
	PRT0DM0 = dm0;
	PRT0DM1 = dm1;
	M8C_SetBank0;
	PRT0DM2 = 0x00;
}


/* ----------------------- */
/*  method                 */
/* ----------------------- */

/* ---------------------------------- */
/* --- touch library initialize */
void ktInitialize(void)
{
	m_bTouchWorking = 0;
	m_nReadingPort = 0;

	/* interrupt mode */
	M8C_SetBank1;
	PRT0IC0 = 0x00;
	PRT0IC1 = 0xff;
	M8C_SetBank0;
	PRT0IE = 0x00;
	/* GPIO setting */
	PRT0DR = 0x00;
	PRT0GS = 0x00;

	Counter16_Charge_Start();
	Counter16_Charge_DisableInt();
}


/* ---------------------------------- */
/* --- touch library start */
void ktStart(void)
{
	m_nReadingPort = 0;
	m_bTouchWorking = 1;
	Counter16_Charge_WritePeriod(48000);
	Counter16_Charge_EnableInt();
	/* GPIO int */
	M8C_EnableIntMask(INT_MSK0, INT_MSK0_GPIO);
	/* refresh */
	M8C_SetBank1;
	PRT0DM0 = 0xff;
	PRT0DM1 = 0x00;
	M8C_SetBank0;
	PRT0DM2 = 0x00;
	PRT0DR = 0x00;
	/* set intrrupt */
	PRT0IE = 0b00000010;	
	/* charge start */
	M8C_SetBank1;
	PRT0DM0 = 0b11111101;
	PRT0DM1 = 0b00000010;
	M8C_SetBank0;
	PRT0DM2 = 0b00000000;
}


/* ---------------------------------- */
/* --- touch library stop */
void ktStop(void)
{
	m_bTouchWorking = 0;
	
	Counter16_Charge_DisableInt();
	/* GPIO int */
	M8C_DisableIntMask(INT_MSK0, INT_MSK0_GPIO);
	
	M8C_SetBank1;
	PRT0DM0 = 0xff;
	PRT0DM1 = 0x00;
	M8C_SetBank0;
	PRT0DM2 = 0x00;
	PRT0DR = 0x00;
	PRT0IE = 0x00;

	m_riseCounter[0] = 0xffff;
	m_riseCounter[1] = 0xffff;
	m_riseCounter[2] = 0xffff;
	m_riseCounter[3] = 0xffff;
}


/* ---------------------------------- */
/* --- read sense value */
unsigned short ktGetValue(char nPort)
{
	return m_riseCounter[nPort];
}


/**
 * get sw value
 * @param nPort [in] port number
 * @return ON/OFF
 */
char ktGetSW(int nPort)
{
	/* === ToDo */
	return 0;
}
